@@ -39,6 +39,10 @@ G_BEGIN_DECLS
 #define GDA_IS_DATA_MODEL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, GDA_TYPE_DATA_MODEL))
 #define GDA_DATA_MODEL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GDA_TYPE_DATA_MODEL, GdaDataModelClass))
 
+/* error reporting */
+extern GQuark gda_data_model_error_quark (void);
+#define GDA_DATA_MODEL_ERROR gda_data_model_error_quark ()
+
 typedef enum {
 	GDA_DATA_MODEL_ACCESS_RANDOM = 1 << 0,
 	GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD = 1 << 1,
@@ -66,7 +70,12 @@ enum {
 	GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
 	GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
 	GDA_DATA_MODEL_VALUES_LIST_ERROR,
-	GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR
+	GDA_DATA_MODEL_VALUE_TYPE_ERROR,
+	GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
+	GDA_DATA_MODEL_ACCESS_ERROR,
+	GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
+	GDA_DATA_MODEL_FILE_EXIST_ERROR,
+	GDA_DATA_MODEL_XML_FORMAT_ERROR
 };
 
 /* struct for the interface */
@@ -80,7 +89,7 @@ struct _GdaDataModelClass {
 	GdaColumn           *(* i_describe_column)  (GdaDataModel *model, gint col);
 	GdaDataModelAccessFlags (* i_get_access_flags) (GdaDataModel *model);
 
-	const GValue        *(* i_get_value_at)     (GdaDataModel *model, gint col, gint row);
+	const GValue        *(* i_get_value_at)     (GdaDataModel *model, gint col, gint row, GError **error);
 	GdaValueAttribute    (* i_get_attributes_at)(GdaDataModel *model, gint col, gint row);
 	GdaDataModelIter    *(* i_create_iter)      (GdaDataModel *model);
 	gboolean             (* i_iter_at_row)      (GdaDataModel *model, GdaDataModelIter *iter, gint row);
@@ -89,6 +98,8 @@ struct _GdaDataModelClass {
 
 	gboolean             (* i_set_value_at)     (GdaDataModel *model, gint col, gint row, 
 						     const GValue *value, GError **error);
+	gboolean             (* i_iter_set_value)   (GdaDataModel *model, GdaDataModelIter *iter, gint col,
+						     const GValue *value, GError **error);
 	gboolean             (* i_set_values)       (GdaDataModel *model, gint row, GList *values,
 						     GError **error);
 	gint                 (* i_append_values)    (GdaDataModel *model, const GList *values, GError **error);
@@ -110,22 +121,21 @@ struct _GdaDataModelClass {
 
 GType               gda_data_model_get_type               (void) G_GNUC_CONST;
 
-gboolean            gda_data_model_is_updatable           (GdaDataModel *model);
-GdaDataModelAccessFlags gda_data_model_get_access_flags       (GdaDataModel *model);
+GdaDataModelAccessFlags gda_data_model_get_access_flags   (GdaDataModel *model);
 
 gint                gda_data_model_get_n_rows             (GdaDataModel *model);
 gint                gda_data_model_get_n_columns          (GdaDataModel *model);
 
 GdaColumn          *gda_data_model_describe_column        (GdaDataModel *model, gint col);
-gint                gda_data_model_get_column_index_by_name(GdaDataModel *model, const gchar *name);
+gint                gda_data_model_get_column_index       (GdaDataModel *model, const gchar *name);
 const gchar        *gda_data_model_get_column_name       (GdaDataModel *model, gint col);
 void                gda_data_model_set_column_name       (GdaDataModel *model, gint col, const gchar *name);
 const gchar        *gda_data_model_get_column_title       (GdaDataModel *model, gint col);
 void                gda_data_model_set_column_title       (GdaDataModel *model, gint col, const gchar *title);
 
-const GValue       *gda_data_model_get_value_at           (GdaDataModel *model, gint col, gint row);
-const GValue       *gda_data_model_get_value_at_col_name  (GdaDataModel *model, 
-							   const gchar *column_name, gint row);
+const GValue       *gda_data_model_get_value_at           (GdaDataModel *model, gint col, gint row, GError **error);
+const GValue       *gda_data_model_get_typed_value_at     (GdaDataModel *model, gint col, gint row, 
+							   GType expected_type, gboolean nullok, GError **error);
 GdaValueAttribute   gda_data_model_get_attributes_at      (GdaDataModel *model, gint col, gint row);
 GdaDataModelIter   *gda_data_model_create_iter            (GdaDataModel *model);
 void                gda_data_model_freeze                 (GdaDataModel *model);
