@@ -21,7 +21,7 @@
  */
 
 #include <stdlib.h>
-#include <glib/gmessages.h>
+#include <glib.h>
 #include <libgda/gda-marshal.h>
 #include <libgda/gda-server-provider.h>
 #include <libgda/gda-server-operation.h>
@@ -144,7 +144,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaServerOperationClass, seq_item_added),
 			      NULL, NULL,
-			      gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
+			      _gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
 			      2, G_TYPE_STRING, G_TYPE_INT);
 	gda_server_operation_signals[SEQUENCE_ITEM_REMOVE] =
 		g_signal_new ("sequence-item-remove",
@@ -152,7 +152,7 @@ gda_server_operation_class_init (GdaServerOperationClass *klass)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GdaServerOperationClass, seq_item_remove),
 			      NULL, NULL,
-			      gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
+			      _gda_marshal_VOID__STRING_INT, G_TYPE_NONE,
 			      2, G_TYPE_STRING, G_TYPE_INT);
 
 	klass->seq_item_added = NULL;
@@ -298,19 +298,14 @@ node_destroy (GdaServerOperation *op, Node *node)
 	case GDA_SERVER_OPERATION_NODE_SEQUENCE: {
 		GSList *list;
 
-		list = node->d.seq.seq_tmpl;
-		while (list) {
+		for (list = node->d.seq.seq_tmpl; list; list = list->next)
 			node_destroy (op, NODE (list->data));
-			list = g_slist_next (list);
-		}
 		g_slist_free (node->d.seq.seq_tmpl);
 
-		list = node->d.seq.seq_items;
-		while (list) {
+		for (list = node->d.seq.seq_items; list; list = list->next)
 			node_destroy (op, NODE (list->data));
-			list = g_slist_next (list);
-		}
 		g_slist_free (node->d.seq.seq_items);
+
 		g_free (node->d.seq.name);
 		g_free (node->d.seq.descr);
 		break;
@@ -318,11 +313,8 @@ node_destroy (GdaServerOperation *op, Node *node)
 	case GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM: {
 		GSList *list;
 
-		list = node->d.seq_item_nodes;
-		while (list) {
+		for (list = node->d.seq_item_nodes; list; list = list->next)
 			node_destroy (op, NODE (list->data));
-			list = g_slist_next (list);
-		}
 		g_slist_free (node->d.seq_item_nodes);
 		break;
 	}
@@ -352,14 +344,15 @@ node_find (GdaServerOperation *op, const gchar *path)
 	if (!path || !*path || (*path != '/'))
 		return NULL;
 
-	list = op->priv->allnodes;
-	while (list && !node) {
+	for (list = op->priv->allnodes; list; list = list->next) {
 		gchar *str;
 		str = node_get_complete_path (op, NODE (list->data));
-		if (!strcmp (str, path))
+		if (!strcmp (str, path)) {
 			node = NODE (list->data);
+			g_free (str);
+			break;
+		}
 		g_free (str);
-		list = list->next;
 	}
 	/*g_print ("%s(%s) => %p\n", __FUNCTION__, path, node);*/
 	return node;
@@ -442,8 +435,7 @@ node_get_complete_path (GdaServerOperation *op, Node *node)
 		return NULL;
 
 	string = g_string_new ("");
-	lnode = node;
-	while (lnode) {
+	for (lnode = node; lnode; lnode = lnode->parent) {
 		if (lnode->type == GDA_SERVER_OPERATION_NODE_SEQUENCE_ITEM) {
 			gchar *str;
 
@@ -456,7 +448,6 @@ node_get_complete_path (GdaServerOperation *op, Node *node)
 		else 
 			g_string_prepend (string, lnode->path_name);
 		g_string_prepend_c (string, '/');
-		lnode = lnode->parent;
 	}
 
 	retval = string->str;
@@ -474,7 +465,7 @@ sequence_add_item (GdaServerOperation *op, Node *node)
 {
 	gchar *path, *seq_path;
 	Node *new_node;
-	GSList *seq_item_nodes;
+	GSList *seq_item_nodes, *list;
 
 	g_assert (node);
 	g_assert (node->type == GDA_SERVER_OPERATION_NODE_SEQUENCE);
@@ -492,6 +483,8 @@ sequence_add_item (GdaServerOperation *op, Node *node)
 	node->d.seq.seq_items = g_slist_append (node->d.seq.seq_items, new_node);
 
 	new_node->d.seq_item_nodes = seq_item_nodes;
+	for (list = seq_item_nodes; list; list = list->next)
+		((Node*) list->data)->parent = new_node;
 
 	clean_nodes_info_cache (op);
 #ifdef GDA_DEBUG_signal
@@ -638,7 +631,7 @@ gda_server_operation_set_property (GObject *object,
 		}
 	}
 
-	if (op->priv->xml_spec_doc && op->priv->cnc_set && op->priv->prov_set) {
+	if (!op->priv->topnodes && op->priv->xml_spec_doc && op->priv->cnc_set && op->priv->prov_set) {
 		/* load XML file */
 		GError *lerror = NULL;
 		op->priv->topnodes = load_xml_spec (op, xmlDocGetRootElement (op->priv->xml_spec_doc), NULL, &lerror);
@@ -702,6 +695,9 @@ xml_validity_error_func (void *ctx, const char *msg, ...)
         g_object_set_data (G_OBJECT (op), "xmlerror", newerr);
 }
 
+/*
+ * Warning: the new nodes' parent is not set!
+ */
 static GSList *
 load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root, GError **error)
 {
@@ -1198,8 +1194,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 	complete_path = node_get_complete_path (op, opnode);
 	switch (opnode->type) {
 	case GDA_SERVER_OPERATION_NODE_PARAMLIST:
-		list = opnode->d.plist->holders;
-		while (list) {
+		for (list = opnode->d.plist->holders; list; list = list->next) {
 			gchar *path;
 			const GValue *value;
 			gchar *str;
@@ -1213,14 +1208,12 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 				else
 					str = gda_value_stringify (value);
 			}
-			node = xmlNewChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
+			node = xmlNewTextChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
 			g_free (str);
 
 			path = g_strdup_printf ("%s/%s", complete_path, gda_holder_get_id (GDA_HOLDER (list->data)));
 			xmlSetProp(node, (xmlChar*)"path", (xmlChar*)path);
 			g_free (path);
-
-			list = g_slist_next (list);
 		}
 		break;
 	case GDA_SERVER_OPERATION_NODE_DATA_MODEL:
@@ -1242,7 +1235,7 @@ node_save (GdaServerOperation *op, Node *opnode, xmlNodePtr parent)
 			else
 				str = gda_value_stringify (value);
 		}
-		node = xmlNewChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
+		node = xmlNewTextChild (parent, NULL, BAD_CAST "op_data", (xmlChar*)str);
 		g_free (str);
 		xmlSetProp(node, (xmlChar*)"path", (xmlChar*)complete_path);
 		break;
@@ -1371,7 +1364,7 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 				switch (opnode->type) {
 				case GDA_SERVER_OPERATION_NODE_PARAMLIST:
 					if (!extension) {
-						g_set_error (error, 0, 0,
+						g_set_error (error, 0, 0, "%s", 
 							     _("Parameterlist values can only be set for individual parameters within it"));
 						allok = FALSE;
 					}
@@ -1428,7 +1421,7 @@ gda_server_operation_load_data_from_xml (GdaServerOperation *op, xmlNodePtr node
 				return FALSE;
 		}
 		else {
-			g_set_error (error, 0, 0,
+			g_set_error (error, 0, 0, "%s", 
 				     _("Missing attribute named 'path'"));
 			return FALSE;
 		}
@@ -1457,12 +1450,9 @@ gda_server_operation_get_root_nodes (GdaServerOperation *op)
 	g_return_val_if_fail (GDA_IS_SERVER_OPERATION (op), NULL);
 	g_return_val_if_fail (op->priv, NULL);	
 
-	list = op->priv->topnodes;
-	retval = g_new0 (gchar *, g_slist_length (list) + 1);
-	while (list) {
+	retval = g_new0 (gchar *, g_slist_length (op->priv->topnodes) + 1);
+	for (list = op->priv->topnodes; list; list = list->next)
 		retval [i++] = node_get_complete_path (op, NODE (list->data));
-		list = g_slist_next (list);
-	}
 
 	return retval;
 }
@@ -1609,11 +1599,8 @@ gda_server_operation_get_sequence_item_names (GdaServerOperation *op, const gcha
 		list = node->d.seq_item_nodes;
 	i = 0;
 	retval = g_new0 (gchar *, g_slist_length (list) + 1);
-	while (list) {
+	for (; list; list = list->next, i++)
 		retval [i] = node_get_complete_path (op, NODE (list->data));
-		i++;
-		list = g_slist_next (list);
-	}
 
 	return retval;
 }
@@ -1990,7 +1977,7 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 		switch (opnode->type) {
 		case GDA_SERVER_OPERATION_NODE_PARAMLIST:
 			if (!extension) {
-				g_set_error (error, 0, 0,
+				g_set_error (error, 0, 0, "%s", 
 					     _("Parameterlist values can only be set for individual parameters within it"));
 				allok = FALSE;
 			}
