@@ -418,7 +418,7 @@ gda_capi_provider_get_server_version (GdaServerProvider *provider, GdaConnection
 /*
  * Get database request
  *
- * Returns the server version as a string, which should be stored in @cnc's associated CapiConnectionData structure
+ * Returns the database name as a string, which should be stored in @cnc's associated CapiConnectionData structure
  */
 static const gchar *
 gda_capi_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc)
@@ -591,7 +591,7 @@ gda_capi_provider_perform_operation (GdaServerProvider *provider, GdaConnection
 	/* If asynchronous connection opening is not supported, then exit now */
 	if (async_cb) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_METHOD_NON_IMPLEMENTED_ERROR,
-			     _("Provider does not support asynchronous server operation"));
+			     "%s", _("Provider does not support asynchronous server operation"));
                 return FALSE;
 	}
 
@@ -824,7 +824,7 @@ gda_capi_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnect
 	    (type == G_TYPE_UCHAR) ||
 	    (type == G_TYPE_ULONG) ||
 	    (type == G_TYPE_UINT) ||
-	    (type == G_TYPE_UINT64)) 
+	    (type == G_TYPE_UINT64))
 		return "integer";
 
 	if ((type == GDA_TYPE_BINARY) ||
@@ -841,7 +841,8 @@ gda_capi_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnect
 	    (type == G_TYPE_STRING) ||
 	    (type == GDA_TYPE_TIME) ||
 	    (type == GDA_TYPE_TIMESTAMP) ||
-	    (type == G_TYPE_INVALID))
+	    (type == G_TYPE_INVALID) ||
+	    (type == G_TYPE_GTYPE))
 		return "string";
 
 	if ((type == G_TYPE_DOUBLE) ||
@@ -907,6 +908,7 @@ gda_capi_provider_statement_prepare (GdaServerProvider *provider, GdaConnection
 				     GdaStatement *stmt, GError **error)
 {
 	GdaCapiPStmt *ps;
+	gboolean retval = FALSE;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
@@ -917,14 +919,58 @@ gda_capi_provider_statement_prepare (GdaServerProvider *provider, GdaConnection
 	if (ps)
 		return TRUE;
 
+	/* render as SQL understood by the provider */
+	GdaSet *params = NULL;
+	gchar *sql;
+	GSList *used_params = NULL;
+	if (! gda_statement_get_parameters (stmt, &params, error))
+                return FALSE;
+        sql = gda_capi_provider_statement_to_sql (provider, cnc, stmt, params, GDA_STATEMENT_SQL_PARAMS_AS_UQMARK,
+						  &used_params, error);
+        if (!sql) 
+		goto out;
+
 	/* prepare @stmt using the C API, creates @ps */
 	TO_IMPLEMENT;
-	if (!ps)
-		return FALSE;
-	else {
-		gda_connection_add_prepared_statement (cnc, stmt, (GdaPStmt *) ps);
-		return TRUE;
-	}
+
+	/* make a list of the parameter names used in the statement */
+	GSList *param_ids = NULL;
+        if (used_params) {
+                GSList *list;
+                for (list = used_params; list; list = list->next) {
+                        const gchar *cid;
+                        cid = gda_holder_get_id (GDA_HOLDER (list->data));
+                        if (cid) {
+                                param_ids = g_slist_append (param_ids, g_strdup (cid));
+                                /*g_print ("PREPARATION: param ID: %s\n", cid);*/
+                        }
+                        else {
+                                g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_PREPARE_STMT_ERROR,
+                                             "%s", _("Unnamed parameter is not allowed in prepared statements"));
+                                g_slist_foreach (param_ids, (GFunc) g_free, NULL);
+                                g_slist_free (param_ids);
+                                goto out;
+                        }
+                }
+        }
+	
+	/* create a prepared statement object */
+	/*ps = gda_capi_pstmt_new (...);*/
+	gda_pstmt_set_gda_statement (_GDA_PSTMT (ps), stmt);
+        _GDA_PSTMT (ps)->param_ids = param_ids;
+        _GDA_PSTMT (ps)->sql = sql;
+
+	gda_connection_add_prepared_statement (cnc, stmt, (GdaPStmt *) ps);
+	g_object_unref (ps);
+
+	retval = TRUE;
+
+ out:
+	if (used_params)
+                g_slist_free (used_params);
+        if (params)
+                g_object_unref (params);
+	return retval;
 }
 
 /*
@@ -950,6 +996,9 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 {
 	GdaCapiPStmt *ps;
 	CapiConnectionData *cdata;
+	gboolean allow_noparam;
+        gboolean empty_rs = FALSE; /* TRUE when @allow_noparam is TRUE and there is a problem with @params
+                                      => resulting data model will be empty (0 row) */
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
@@ -958,10 +1007,21 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 	/* If asynchronous connection opening is not supported, then exit now */
 	if (async_cb) {
 		g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_METHOD_NON_IMPLEMENTED_ERROR,
-			     _("Provider does not support asynchronous statement execution"));
-                return FALSE;
+			     "%s", _("Provider does not support asynchronous statement execution"));
+                return NULL;
 	}
 
+	if (! (model_usage & GDA_STATEMENT_MODEL_RANDOM_ACCESS) &&
+            ! (model_usage & GDA_STATEMENT_MODEL_CURSOR_FORWARD))
+                model_usage |= GDA_STATEMENT_MODEL_RANDOM_ACCESS;
+	
+        allow_noparam = (model_usage & GDA_STATEMENT_MODEL_ALLOW_NOPARAM) &&
+                (gda_statement_get_statement_type (stmt) == GDA_SQL_STATEMENT_SELECT);
+	
+        if (last_inserted_row)
+                *last_inserted_row = NULL;
+
+	/* Get private data */
 	cdata = (CapiConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata) 
 		return FALSE;
@@ -975,13 +1035,20 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			 * where the C API cannot allow them (for example if the variable is the table name
 			 * in a SELECT statement). The action here is to get the actual SQL code for @stmt,
 			 * and use that SQL instead of @stmt to create another GdaCapiPStmt object.
+			 *
+			 * Don't call gda_connection_add_prepared_statement() with this new prepared statement
+			 * as it will be destroyed once used.
 			 */
 			TO_IMPLEMENT;
 			return NULL;
 		}
-		else
+		else {
 			ps = (GdaCapiPStmt *) gda_connection_get_prepared_statement (cnc, stmt);
+			g_object_ref (ps);
+		}
 	}
+	else
+		g_object_ref (ps);
 	g_assert (ps);
 
 	/* optionnally reset the prepared statement if required by the API */
@@ -1001,7 +1068,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			gda_connection_event_set_description (event, _("Missing parameter(s) to execute query"));
 			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR,
-				     _("Missing parameter(s) to execute query"));
+				     "%s", _("Missing parameter(s) to execute query"));
 			break;
 		}
 
@@ -1014,24 +1081,41 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			}
 		}
 		if (!h) {
-			gchar *str;
-			str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
-			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
-			gda_connection_event_set_description (event, str);
-			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
-				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
-			g_free (str);
-			break;
+			if (! allow_noparam) {
+				gchar *str;
+				str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
+				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (event, str);
+				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
+				g_free (str);
+				break;
+			}
+			else {
+                                /* bind param to NULL */
+                                TO_IMPLEMENT;
+                                empty_rs = TRUE;
+                                continue;
+                        }
+
 		}
 		if (!gda_holder_is_valid (h)) {
-			gchar *str;
-			str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
-			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
-			gda_connection_event_set_description (event, str);
-			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
-				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, str);
-			g_free (str);
-			break;
+			if (! allow_noparam) {
+				gchar *str;
+				str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
+				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (event, str);
+				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
+				g_free (str);
+				break;
+			}
+			else {
+                                /* bind param to NULL */
+				TO_IMPLEMENT;
+                                empty_rs = TRUE;
+                                continue;
+                        }
 		}
 
 		/* actual binding using the C API, for parameter at position @i */
@@ -1041,6 +1125,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 		
 	if (event) {
 		gda_connection_add_event (cnc, event);
+		g_object_unref (ps);
 		return NULL;
 	}
 	
@@ -1048,6 +1133,36 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 	event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
         gda_connection_event_set_description (event, _GDA_PSTMT (ps)->sql);
         gda_connection_add_event (cnc, event);
+
+	if (empty_rs) {
+		/* There are some missing parameters, so the SQL can't be executed but we still want
+		 * to execute something to get the columns correctly. A possibility is to actually
+		 * execute another SQL which is the code shown here.
+		 *
+		 * To adapt depending on the C API and its features */
+		GdaStatement *estmt;
+                gchar *esql;
+                estmt = gda_select_alter_select_for_empty (stmt, error);
+                if (!estmt) {
+			g_object_unref (ps);
+                        return NULL;
+		}
+                esql = gda_statement_to_sql (estmt, NULL, error);
+                g_object_unref (estmt);
+                if (!esql) {
+			g_object_unref (ps);
+                        return NULL;
+		}
+
+		/* Execute the 'esql' SQL code */
+                g_free (esql);
+
+		TO_IMPLEMENT;
+	}
+	else {
+		/* Execute the _GDA_PSTMT (ps)->sql SQL code */
+		TO_IMPLEMENT;
+	}
 	
 	/* execute prepared statement using C API depending on its kind */
 	if (! g_ascii_strncasecmp (_GDA_PSTMT (ps)->sql, "SELECT", 6) ||
@@ -1062,6 +1177,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 
                 data_model = (GObject *) gda_capi_recordset_new (cnc, ps, params, flags, col_types);
 		gda_connection_internal_statement_executed (cnc, stmt, params, NULL); /* required: help @cnc keep some stats */
+		g_object_unref (ps);
 		return data_model;
         }
 	else {
@@ -1072,6 +1188,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 		/* Create GdaConnectionEvent notice with the type of command and impacted rows */
 
 		gda_connection_internal_statement_executed (cnc, stmt, params, event); /* required: help @cnc keep some stats */
+		g_object_unref (ps);
 		return (GObject*) set;
 	}
 }
